from grapi.grapi import Grapi


def test_get():
    api = Grapi("http://echo.jsontest.com/one/two", "l3fkjh345ksdfjll345")
    result = api._get()
    expected = "{\"one\": \"two\"}\n"
    assert result.text == expected

# TODO: Write tests for all the things



import requests
from urllib.parse import urlparse
from .endpoints import Endpoints


class Grapi:
    def __init__(self, url, access_token):

        self.url = url
        self._endpoint = urlparse(self.url).path
        self._access_token = (access_token, "token")
        self.methods = {
            "get": self._get,
            "post": self._post,
            "put": self._put,
            "delete": self._delete
        }

    def _get(self, **kwargs):
        response = requests.get(self.url, auth=self._access_token, params=kwargs)
        return response

    def _post(self, **kwargs):
        # TODO: Implement
        raise NotImplementedError("POST: Not Yet Implemented")

    def _put(self, **kwargs):
        # TODO: Implement
        raise NotImplementedError("PUT: Not Yet Implemented")

    def _delete(self, **kwargs):
        # TODO: Implement
        raise NotImplementedError("DELETE: Not Yet Implemented")

    def _methods(self, method):
        # Gets a pointer to the right function depending on API access method
        if method not in self.methods.keys():
            raise ValueError("\"{0}\" is not a supported method. " +
                             "Please use one of the following: {1}".format(method, self.methods.keys()))
        return self.methods[method]

    def send(self, method, **kwargs):
        endpoints = Endpoints()
        keys = kwargs.keys()
        print(self._endpoint)
        endpoints.check(keys, self._endpoint)
        return self._methods(method)(**kwargs)










class Endpoints:
    # This class contains a list of the REQUIRED keyword arguments for each API url endpoint that's implemented.
    def __init__(self):
        # TODO: Allow temporarily adding new endpoints by passing in a dict.
        self._endpoints = {
            "/api/search/universal/absolute": ["query", "from", "to", "fields"],
            "/api/search/universal/absolute/terms": ["query", "from", "to", "fields"],
            "/api/search/universal/relative": ["query", "range", "fields"],
            "/api/search/universal/relative/terms": ["query", "range", "fields"]
        }

    def check(self, keys, endpoint):
        print(endpoint)
        print(self._endpoints[endpoint])
        if not set(self._endpoints[endpoint]).issubset(set(keys)):
            raise ValueError("Minimum required arguments missing for this API endpoint.\n" +
                             "Given: {0}\n" +
                             "Required: {1}\n".format(keys, self._endpoints))
        else:
            return True



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="grapi",
    version="0.1.0",
    author="Andrew Craighead",
    author_email="craigheadap@missouri.edu",
    description="Python 3 bindings for Graylog's REST API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/interputed/grapi",
    packages=setuptools.find_packages(),
    keywords=["graylog", "graylog-api", "graylog api", "python3 graylog", "python3 graylog api", "python3 graylog-api"],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
    ],
)